# SPDX-FileCopyrightText: 2024 Contributors to the Fedora Project
#
# SPDX-License-Identifier: LGPL-3.0-or-later

from fedora_messaging import message

SCHEMA_URL = "http://fedoraproject.org/message-schema/"


class _PublishedV1(message.Message):
    """
    Base class that just defines a base topic and common properties.

    This message class should never be used to send a message, use a sub-class.
    """

    topic = ".".join(["fedora_image_uploader", "published", "v1"])

    @property
    def app_name(self):
        return "fedora-image-uploader"


class AwsPublishedV1(_PublishedV1):
    """Published when an AWS AMI is created from an image."""

    topic = ".".join([_PublishedV1.topic, "aws"])
    body_schema = {
        "id": f"{SCHEMA_URL}/v1/{'.'.join([_PublishedV1.topic, 'aws'])}",
        "$schema": "https://json-schema.org/draft/2019-09/schema",
        "description": (
            "Schema for messages sent by fedora-image-uploader when a "
            "new Amazon Web Services image is published."
        ),
        "type": "object",
        "properties": {
            "architecture": {
                "type": "string",
                "description": "The machine architecture of the image (x86_64, aarch64, etc).",
            },
            "compose_id": {
                "type": "string",
                "description": "The compose ID this image was created from.",
            },
            "image_name": {
                "type": "string",
                "description": "The name of the AMI.",
            },
            "regions": {
                "type": "object",
                "description": (
                    "A map of regions to AMI IDs. The object keys are the AWS region and "
                    "the value is the AMI ID."
                ),
            },
            "release": {
                "type": "integer",
                "description": "The release number associated with the image.",
            },
            "subvariant": {
                "type": "string",
                "description": "The subvariant of the image (e.g. Cloud_Base).",
            },
        },
        "required": [
            "architecture",
            "compose_id",
            "image_name",
            "regions",
            "release",
            "subvariant",
        ],
    }

    @property
    def summary(self):
        return (
            f"{self.app_name} published AWS images from compose {self.body['compose_id']} as "
            f"{self.body['image_name']} in {len(self.body['regions'])} regions"
        )

    def __str__(self):
        regions_and_ids = [f"{region} as {id}" for region, id in self.body["regions"].items()]
        return (
            "A new image has been published to Amazon Web Services:\n\n"
            f"\tArchitecture: {self.body['architecture']}\n"
            f"\tCompose ID: {self.body['compose_id']}\n"
            f"\tImage Name: {self.body['image_name']}\n"
            f"\tRegions: {', '.join(regions_and_ids)}\n"
        )


class AzurePublishedV1(_PublishedV1):
    """
    Published when an image is uploaded to the Azure image gallery.
    """

    topic = ".".join([_PublishedV1.topic, "azure"])
    body_schema = {
        "id": f"{SCHEMA_URL}/v1/{'.'.join([_PublishedV1.topic, 'azure'])}",
        "$schema": "https://json-schema.org/draft/2019-09/schema",
        "description": (
            "Schema for messages sent by fedora-image-uploader when a "
            "new Azure image is published."
        ),
        "type": "object",
        "properties": {
            "architecture": {
                "type": "string",
                "description": "The machine architecture of the image (x86_64, aarch64, etc).",
            },
            "compose_id": {
                "type": "string",
                "description": "The compose ID this image was created from.",
            },
            "image_definition_name": {
                "type": "string",
                "description": "The name of the image collection within the Azure gallery.",
            },
            "image_version_name": {
                "type": "string",
                "description": (
                    "The image version name which uniquely identifies it "
                    "in the image definition."
                ),
            },
            "image_resource_id": {
                "type": "string",
                "description": (
                    "The Azure resource ID of the image which can be used to "
                    "provision a virtual machine."
                ),
            },
            "regions": {
                "type": "array",
                "description": (
                    "The regions to which the image was replicated; virtual machines using this "
                    "image must be launched in one of these regions."
                ),
                "items": {"type": "string"},
            },
            "release": {
                "type": "integer",
                "description": "The release number associated with the image.",
            },
            "subvariant": {
                "type": "string",
                "description": "The subvariant of the image (e.g. Cloud_Base).",
            },
        },
        "required": [
            "architecture",
            "compose_id",
            "image_definition_name",
            "image_version_name",
            "image_resource_id",
            "regions",
            "release",
            "subvariant",
        ],
    }

    @property
    def summary(self):
        return (
            f"{self.app_name} published Azure image from compose {self.body['compose_id']} as "
            f"version {self.body['image_version_name']} to {self.body['image_definition_name']}"
        )

    def __str__(self):
        return (
            "A new image has been published to the Azure image gallery:\n\n"
            f"\tArchitecture: {self.body['architecture']}\n"
            f"\tCompose ID: {self.body['compose_id']}\n"
            f"\tImage Definition Name: {self.body['image_definition_name']}\n"
            f"\tImage Version Name: {self.body['image_version_name']}\n"
            f"\tImage Resource ID: {self.body['image_resource_id']}\n"
            f"\tRegions: {', '.join(self.body['regions'])}\n"
        )


class ContainerPublishedV1(_PublishedV1):
    """Published when a new image manifest is pushed."""

    topic = ".".join([_PublishedV1.topic, "container"])
    body_schema = {
        "id": f"{SCHEMA_URL}/v1/{'.'.join([_PublishedV1.topic, 'container'])}",
        "$schema": "https://json-schema.org/draft/2019-09/schema",
        "description": (
            "Schema for messages sent by fedora-image-uploader when a new "
            "container manifest is uploaded."
        ),
        "type": "object",
        "properties": {
            "architectures": {
                "type": "array",
                "description": "The machine architectures of the images (x86_64, aarch64, etc).",
                "items": {"type": "string"},
            },
            "compose_id": {
                "type": "string",
                "description": "The compose ID this image was created from.",
            },
            "registries": {
                "type": "array",
                "items": {"type": "string"},
                "description": "The registries where the container was published.",
            },
            "release": {
                "type": "integer",
                "description": "The release number associated with the image.",
            },
            "repository": {
                "type": "string",
                "description": "The repository where the container was published.",
            },
            "tags": {
                "type": "array",
                "items": {"type": "string"},
                "description": (
                    "The names under which the manifest was published (e.g. '40', 'latest')"
                ),
            },
        },
        "required": ["architectures", "compose_id", "registries", "release", "repository", "tags"],
    }

    @property
    def containers(self):
        """List of containers affected by the action that generated this message."""
        containers = []
        for registry in self.body["registries"]:
            for tag in self.body["tags"]:
                containers.append(f"{registry}/{self.body['repository']}:{tag}")
        return containers

    @property
    def summary(self):
        return (
            f"{self.app_name} published container manifest from compose {self.body['compose_id']} "
            f"to {', '.join(self.body['registries'])}."
        )

    def __str__(self):
        return (
            "A new container manifest has been published:\n\n"
            f"\tCompose ID: {self.body['compose_id']}\n"
            f"\tRegistries: {', '.join(self.body['registries'])}\n"
            f"\tRepository: {self.body['repository']}\n"
            f"\tTags: {', '.join(self.body['tags'])}\n"
            f"\tArchitectures: {', '.join(self.body['architectures'])}\n"
        )


class GcpPublishedV1(_PublishedV1):
    topic = ".".join([_PublishedV1.topic, "gcp"])
    body_schema = {
        "id": f"{SCHEMA_URL}/v1/{'.'.join([_PublishedV1.topic, 'gcp'])}",
        "$schema": "https://json-schema.org/draft/2019-09/schema",
        "description": (
            "Schema for messages sent by fedora-image-uploader when a "
            "new Google Cloud Platform image is published."
        ),
        "type": "object",
        "properties": {
            "architecture": {
                "type": "string",
                "description": "The machine architecture of the image (x86_64, aarch64, etc).",
            },
            "compose_id": {
                "type": "string",
                "description": "The compose ID this image was created from.",
            },
            "release": {
                "type": "integer",
                "description": "The release number associated with the image.",
            },
            "subvariant": {
                "type": "string",
                "description": "The subvariant of the image (e.g. Cloud_Base).",
            },
            "family": {
                "type": "string",
                "description": "The Google Compute Engine OS family for the image.",
            },
            "image_name": {
                "type": "string",
                "description": "The name of the image.",
            },
            "image_url": {
                "type": "string",
                "description": "The URL of the image in Google Cloud Engine.",
            },
            "storage_locations": {
                "type": "array",
                "items": {"type": "string"},
                "description": "The geographic location codes where the image is available.",
            },
        },
        "required": [
            "architecture",
            "compose_id",
            "release",
            "subvariant",
            "family",
            "image_name",
            "image_url",
            "storage_locations",
        ],
    }

    @property
    def summary(self):
        return (
            f"{self.app_name} published the {self.body['architecture']} image from compose"
            f" {self.body['compose_id']} to the {self.body['family']} family in Google"
            " Cloud Platform"
        )

    def __str__(self):
        return (
            "A new image has been published to Google Cloud Platform:\n\n"
            f"\tArchitecture: {self.body['architecture']}\n"
            f"\tCompose ID: {self.body['compose_id']}\n"
            f"\tFamily: {self.body['family']}\n"
            f"\tImage Name: {self.body['image_name']}\n"
            f"\tImage URL: {self.body['image_url']}\n"
        )



# SPDX-FileCopyrightText: 2024 Contributors to the Fedora Project
#
# SPDX-License-Identifier: LGPL-3.0-or-later

"""Unit tests for the message schema."""

import pytest
from jsonschema import ValidationError

from fedora_image_uploader_messages import (
    AwsPublishedV1,
    AzurePublishedV1,
    ContainerPublishedV1,
    GcpPublishedV1,
)


def test_aws_schema():
    """
    Assert the message schema validates a message with the required fields.
    """
    body = {
        "architecture": "x86_64",
        "compose_id": "Fedora-40-20240501.n.0",
        "image_name": "Fedora-Cloud-AmazonEC2",
        "regions": {
            "us-east-2": "ami-123",
            "us-west-1": "ami-321",
        },
        "release": 40,
        "subvariant": "Cloud_Base",
    }
    message = AwsPublishedV1(body=body)
    message.validate()


def test_aws_missing_fields():
    """Assert an exception is actually raised on validation failure."""
    body = {
        "architecture": "x86_64",
        "compose_id": "Fedora-40-20240501.n.0",
        "image_name": "Fedora-Cloud-AmazonEC2",
        "regions": {
            "us-east-2": "ami-123",
            "us-west-1": "ami-321",
        },
        "release": 40,
        "subvariant": "Cloud_Base",
    }
    for key in body:
        missing_body = body.copy()
        del missing_body[key]
        message = AwsPublishedV1(body=missing_body)
        with pytest.raises(ValidationError):
            message.validate()


def test_aws_display():
    """Assert the human-readable version is as expected."""
    body = {
        "architecture": "x86_64",
        "compose_id": "Fedora-40-20240501.n.0",
        "image_name": "Fedora-Cloud-AmazonEC2",
        "regions": {
            "us-east-2": "ami-123",
            "us-west-1": "ami-321",
        },
        "release": 40,
        "subvariant": "Cloud_Base",
    }
    message = AwsPublishedV1(body=body)
    message.validate()

    assert message.summary == (
        "fedora-image-uploader published AWS images from compose Fedora-40-20240501.n.0"
        " as Fedora-Cloud-AmazonEC2 in 2 regions"
    )
    assert str(message) == (
        """A new image has been published to Amazon Web Services:

\tArchitecture: x86_64
\tCompose ID: Fedora-40-20240501.n.0
\tImage Name: Fedora-Cloud-AmazonEC2
\tRegions: us-east-2 as ami-123, us-west-1 as ami-321
"""
    )


def test_azure_schema():
    """
    Assert the message schema validates a message with the required fields.
    """
    body = {
        "architecture": "x86_64",
        "compose_id": "Fedora-40-20240501.n.0",
        "image_definition_name": "Fedora-Cloud-40-x64",
        "image_version_name": "40.20240501.0",
        "image_resource_id": (
            "/CommunityGalleries/Fedora-5e266ba4-2250-406d-adad-5d73860d958f/Images"
            "/Fedora-Cloud-40-x64/Versions/40.20240501.0"
        ),
        "regions": ["eastus", "eastus2", "centralus"],
        "release": 40,
        "subvariant": "Cloud_Base",
    }
    message = AzurePublishedV1(body=body)
    message.validate()


def test_azure_missing_fields():
    """Assert an exception is actually raised on validation failure."""
    body = {
        "architecture": "x86_64",
        "compose_id": "Fedora-40-20240501.n.0",
        "image_definition_name": "Fedora-Cloud-40-x64",
        "image_version_name": "40.20240501.0",
        "image_resource_id": (
            "/CommunityGalleries/Fedora-5e266ba4-2250-406d-adad-5d73860d958f/Images"
            "/Fedora-Cloud-40-x64/Versions/40.20240501.0"
        ),
        "regions": ["eastus", "eastus2", "centralus"],
        "release": 40,
        "subvariant": "Cloud_Base",
    }
    for key in body:
        missing_body = body.copy()
        del missing_body[key]
        message = AzurePublishedV1(body=missing_body)
        with pytest.raises(ValidationError):
            message.validate()


def test_azure_str():
    """Assert __str__ and the summary property produce a useful messages."""
    body = {
        "architecture": "x86_64",
        "compose_id": "Fedora-40-20240501.n.0",
        "image_definition_name": "Fedora-Cloud-40-x64",
        "image_version_name": "40.20240501.0",
        "image_resource_id": (
            "/CommunityGalleries/Fedora-5e266ba4-2250-406d-adad-5d73860d958f/Images"
            "/Fedora-Cloud-40-x64/Versions/40.20240501.0"
        ),
        "regions": ["eastus", "eastus2", "centralus"],
        "release": 40,
        "subvariant": "Cloud_Base",
    }

    expected_summary = (
        "fedora-image-uploader published Azure image from compose Fedora-40-20240501.n.0 as "
        "version 40.20240501.0 to Fedora-Cloud-40-x64"
    )
    expected_str = (
        "A new image has been published to the Azure image gallery:\n\n"
        "\tArchitecture: x86_64\n"
        "\tCompose ID: Fedora-40-20240501.n.0\n"
        "\tImage Definition Name: Fedora-Cloud-40-x64\n"
        "\tImage Version Name: 40.20240501.0\n"
        "\tImage Resource ID: /CommunityGalleries/Fedora-5e266ba4-2250-406d-adad-5d73860d958f/"
        "Images/Fedora-Cloud-40-x64/Versions/40.20240501.0\n"
        "\tRegions: eastus, eastus2, centralus\n"
    )
    message = AzurePublishedV1(body=body)
    message.validate()

    assert expected_summary == message.summary
    assert expected_str == str(message)


def test_container_str():
    """Assert the container summary is correct."""
    body = {
        "architectures": ["x86_64", "ppc64le", "aarch64", "s390x"],
        "compose_id": "Fedora-40-20240501.n.0",
        "registries": ["quay.io/fedora", "registry.fedoraproject.org"],
        "repository": "fedora",
        "tags": ["40", "latest"],
    }

    expected_summary = (
        "fedora-image-uploader published container manifest from compose Fedora-40-20240501.n.0 "
        "to quay.io/fedora, registry.fedoraproject.org."
    )
    expected_str = (
        "A new container manifest has been published:\n\n"
        "\tCompose ID: Fedora-40-20240501.n.0\n"
        "\tRegistries: quay.io/fedora, registry.fedoraproject.org\n"
        "\tRepository: fedora\n"
        "\tTags: 40, latest\n"
        "\tArchitectures: x86_64, ppc64le, aarch64, s390x\n"
    )

    message = ContainerPublishedV1(body=body)
    assert expected_summary == message.summary
    assert expected_str == str(message)


def test_gcp_str():
    message = GcpPublishedV1(
        topic="fedora_image_uploader.published.v1.gcp.rc.Cloud_Base.aarch64",
        body={
            "architecture": "x86_64",
            "compose_id": "Fedora-40-20240414.0",
            "release": 40,
            "subvariant": "Cloud_Base",
            "family": "fedora-cloud-40",
            "image_name": "fedora-cloud-40-1-14-x86-64",
            "image_url": "https://example.com/link",
            "storage_locations": ["us"],
        },
    )
    expected_summary = (
        "fedora-image-uploader published the x86_64 image from compose Fedora-40-20240414.0 to "
        "the fedora-cloud-40 family in Google Cloud Platform"
    )
    expected_str = (
        "A new image has been published to Google Cloud Platform:\n\n"
        "\tArchitecture: x86_64\n"
        "\tCompose ID: Fedora-40-20240414.0\n"
        "\tFamily: fedora-cloud-40\n"
        "\tImage Name: fedora-cloud-40-1-14-x86-64\n"
        "\tImage URL: https://example.com/link\n"
    )

    assert message.summary == expected_summary
    assert str(message) == expected_str
